CONN_UNIFIED_MAPPINGS = {
    "db2": {"datasource_type": "db2", "credential_key": "[db2-conn|cloud]", "label": "IBM Db2"},
    "db2cloud": {"datasource_type": "db2cloud", "credential_key": "[db2cloud-conn]", "label": "IBM Db2 on Cloud"},
    "bigsql": {"datasource_type": "bigsql", "credential_key": "[db2BigSql|cloud]", "label": "IBM Db2 Big SQL"},
    "db2iseries": {"datasource_type": "db2iseries", "credential_key": "[db2I-conn|cloud]", "label": "IBM Db2 for i"},
    "db2zos": {"datasource_type": "db2zos", "credential_key": "[db2zos-conn|cloud]", "label": "IBM Db2 for z/OS"},
    "amazons3": {"datasource_type": "amazons3", "credential_key": "[s3-amazon]", "label": "Amazon S3"},
    "mysql": {"datasource_type": "mysql", "credential_key": "[mysql-conn|cloud]", "label": "MySQL"},
    "datastax": {
        "datasource_type": "datastax",
        "credential_key": "[datastax-conn|cloud]",
        "label": "DataStax Enterprise",
    },
    "informix": {"datasource_type": "informix", "credential_key": "[Informix-conn|cloud]", "label": "IBM Informix"},
    "match360": {"datasource_type": "match360", "credential_key": "[match360-conn]", "label": "IBM Match 360"},
    "dvm": {
        "datasource_type": "dvm",
        "credential_key": "[dvm|cloud]",
        "label": "IBM Data Virtualization Manager for z/OS",
    },
    "dv": {"datasource_type": "dv", "credential_key": "[dv-onPrem|onPrem]", "label": "IBM Data Virtualization"},
    "generics3": {"datasource_type": "generics3", "credential_key": "[s3-generic]", "label": "Generic S3"},
    "box": {"datasource_type": "box", "credential_key": "[box-conn]", "label": "Box"},
    "elasticsearch": {
        "datasource_type": "elasticsearch",
        "credential_key": "[elasticsearch-conn]",
        "label": "Elasticsearch",
    },
    "dropbox": {"datasource_type": "dropbox", "credential_key": "[dropbox-conn]", "label": "Dropbox"},
    "ftp": {"datasource_type": "ftp", "credential_key": "[ftp-conn|cloud]", "label": "FTP"},
    "tableau": {"datasource_type": "tableau", "credential_key": "[tableau-cloud-conn]", "label": "Tableau"},
    "cassandra": {
        "datasource_type": "cassandra",
        "credential_key": "[cassandra-conn|cloud]",
        "label": "Apache Cassandra",
    },
    "impala": {"datasource_type": "impala", "credential_key": "[impala-conn|cloud]", "label": "Apache Impala"},
    "derby": {"datasource_type": "derby", "credential_key": "[apacheDerby-conn|cloud]", "label": "Apache Derby"},
    "bigquery": {"datasource_type": "bigquery", "credential_key": "[bigquery-conn]", "label": "Google BigQuery"},
    "azuredatalake": {
        "datasource_type": "azuredatalake",
        "credential_key": "[azure-datalake-conn]",
        "label": "Microsoft Azure Data Lake Storage",
    },
    "azuresql": {
        "datasource_type": "azuresql",
        "credential_key": "[azure-sql-conn]",
        "label": "Microsoft Azure SQL Database",
    },
    "sqlserver": {
        "datasource_type": "sqlserver",
        "credential_key": "[sqlServer-conn|cloud]",
        "label": "Microsoft SQL Server",
    },
    "sapodata": {"datasource_type": "sapodata", "credential_key": "[sapodata|onPrem]", "label": "SAP OData"},
    "sybase": {"datasource_type": "sybase", "credential_key": "[SAPASE-conn|cloud]", "label": "SAP ASE"},
    "mariadb": {"datasource_type": "mariadb", "credential_key": "[mariadb-conn]", "label": "MariaDB"},
    "postgresql": {"datasource_type": "postgresql", "credential_key": "[postgres-conn]", "label": "PostgreSQL"},
    "greenplum": {"datasource_type": "greenplum", "credential_key": "[greenplum|cloud]", "label": "Greenplum"},
    "oracle": {"datasource_type": "oracle", "credential_key": "[oracle-conn|cloud]", "label": "Oracle"},
    "dremio": {"datasource_type": "dremio", "credential_key": "[Dremio|cloud]", "label": "Dremio"},
    "mongodb": {"datasource_type": "mongodb", "credential_key": "[mongodb-conn]", "label": "MongoDB"},
    "presto": {"datasource_type": "presto", "credential_key": "[presto]", "label": "Presto"},
    "teradata": {"datasource_type": "teradata", "credential_key": "[teradata|cloud]", "label": "Teradata"},
    "netezza": {
        "datasource_type": "netezza",
        "credential_key": "[netezza-conn|cloud]",
        "label": "IBM Netezza Performance Server",
    },
    "denodo": {"datasource_type": "denodo", "credential_key": "[denodo-conn|cloud]", "label": "Denodo"},
    "vertica": {"datasource_type": "vertica", "credential_key": "[vertica-conn]", "label": "Vertica"},
    "snowflake": {"datasource_type": "snowflake", "credential_key": "[snowflake-conn]", "label": "Snowflake"},
    "singlestore": {"datasource_type": "singlestore", "credential_key": "[SingleStore]", "label": "SingleStoreDB"},
    "salesforce": {"datasource_type": "salesforce", "credential_key": "[salesforce-conn]", "label": "Salesforce.com"},
    "azuresynapse": {
        "datasource_type": "azuresynapse",
        "credential_key": "[azure-synapse-conn]",
        "label": "Microsoft Azure Synapse Analytics",
    },
    "db2fordatastage": {
        "datasource_type": "db2-datastage",
        "credential_key": "[db2datastage|cloud]",
        "label": "IBM Db2 for DataStage",
    },
    "amazon_redshift": {"datasource_type": "redshift", "credential_key": "[redshift-conn]", "label": "Amazon Redshift"},
    "mysql_amazon": {
        "datasource_type": "mysql-amazon",
        "credential_key": "[rds-mysql-conn]",
        "label": "Amazon RDS for MySQL",
    },
    "cloud_object_storage": {
        "datasource_type": "cloudobjectstorage",
        "credential_key": "[cos-conn]",
        "label": "IBM Cloud Object Storage",
    },
    "cognos_analytics": {
        "datasource_type": "cognos-analytics",
        "credential_key": "[cognos-conn]",
        "label": "IBM Cognos Analytics",
    },
    "mongodb_ibmcloud": {
        "datasource_type": "mongodb-ibmcloud",
        "credential_key": "[mongodb-conn]",
        "label": "IBM Cloud Databases for MongoDB",
    },
    "hdfs_apache": {
        "datasource_type": "hdfs-apache",
        "credential_key": "[HDFS-SSL-conn|cloud]",
        "label": "Apache HDFS",
    },
    "apache_kafka": {"datasource_type": "apachekafka", "credential_key": "[Kafka-conn]", "label": "Apache Kafka"},
    "cassandra_datastage": {
        "datasource_type": "cassandra-datastage",
        "credential_key": "[cassandraDS-conn|cloud]",
        "label": "Apache Cassandra for DataStage",
    },
    "google_cloud_storage": {
        "datasource_type": "googlecloudstorage",
        "credential_key": "[googlecos-conn]",
        "label": "Google Cloud Storage",
    },
    "google_pub_sub": {
        "datasource_type": "googlepubsub",
        "credential_key": "[pubsub-conn]",
        "label": "Google Cloud Pub/Sub",
    },
    "azure_blob_storage": {
        "datasource_type": "azureblobstorage",
        "credential_key": "[azure-blob-storage-conn]",
        "label": "Microsoft Azure Blob Storage",
    },
    "azure_cosmos": {
        "datasource_type": "cosmos",
        "credential_key": "[cosmos-conn]",
        "label": "Microsoft Azure Cosmos DB",
    },
    "azure_databricks": {
        "datasource_type": "databricks",
        "credential_key": "[azure-databricks-conn]",
        "label": "Microsoft Azure Databricks",
    },
    "azure_file_storage": {
        "datasource_type": "azurefilestorage",
        "credential_key": "[azure-file-storage-conn]",
        "label": "Microsoft Azure File Storage",
    },
    "sapiq": {"datasource_type": "sybaseiq", "credential_key": "[SAP-IQ-conn|cloud]", "label": "SAP IQ"},
    "postgresql_ibmcloud": {
        "datasource_type": "postgresql-ibmcloud",
        "credential_key": "[db-for-postgres-conn]",
        "label": "IBM Cloud Databases for PostgreSQL",
    },
    "teradata_datastage": {
        "datasource_type": "teradata-datastage",
        "credential_key": "[TeradataOptimized|cloud]",
        "label": "Teradata database for DataStage",
    },
    "oracle_datastage": {
        "datasource_type": "oracle-datastage",
        "credential_key": "[OracleDs|cloud]",
        "label": "Oracle Database for DataStage",
    },
    "apache_hive": {"datasource_type": "hive", "credential_key": "[hive-conn|cloud]", "label": "Apache Hive"},
    "azure_postgresql": {
        "datasource_type": "postgresql-azure",
        "credential_key": "[azure-postgres-conn]",
        "label": "Azure PostgreSQL",
    },
    "watsonx_data": {
        "datasource_type": "lakehouse",
        "credential_key": "[watsonxData-conn]",
        "label": "IBM watsonx.data Presto",
    },
    "planning_analytics": {
        "datasource_type": "tm1odata",
        "credential_key": "[ibm-planning-analytics-conn|cloud]",
        "label": "IBM Planning Analytics",
    },
    "odbc": {"datasource_type": "odbc-datastage", "credential_key": "[odbcDb2-conn|cloud]", "label": "ODBC"},
    "ibm_mq": {
        "datasource_type": "webspheremq-datastage",
        "credential_key": "[websphere-tcp-conn|cloud]",
        "label": "IBM MQ",
    },
    "amazonrds_oracle": {
        "datasource_type": "oracle-amazon",
        "credential_key": "[rds-oracle-conn]",
        "label": "Amazon RDS for Oracle",
    },
    "db2warehouse": {
        "datasource_type": "dashdb",
        "credential_key": "[DB2_Warehouse-conn]",
        "label": "IBM Db2 Warehouse",
    },
    "amazon_postgresql": {
        "datasource_type": "postgresql-amazon",
        "credential_key": "[PostgresSQL-AmazonRDS-conn]",
        "label": "Amazon RDS for PostgreSQL",
    },
    "google_looker": {"datasource_type": "looker", "credential_key": "[looker-conn]", "label": "Google Looker"},
    "netezza_optimized": {
        "datasource_type": "netezza-datastage",
        "credential_key": "[netezza-conn|cloud]",
        "label": "IBM Netezza Performance Server for DataStage",
    },
    "jdbc": {"datasource_type": "jdbc", "credential_key": "", "label": "Generic JDBC"},
    "http": {"datasource_type": "http", "credential_key": "", "label": "HTTP"},
    "salesforceapi": {
        "datasource_type": "salesforce-datastage",
        "credential_key": "",
        "label": "Salesforce API for DataStage",
    },
}
