# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from typing import Literal

StageTypeStr = Literal[
    "Address Verification",
    "Aggregator",
    "Amazon RDS for MySQL",
    "Amazon RDS for Oracle",
    "Amazon RDS for PostgreSQL",
    "Amazon Redshift",
    "Amazon S3",
    "Apache Cassandra",
    "Apache Cassandra for DataStage",
    "Apache Derby",
    "Apache HBase",
    "Apache HDFS",
    "Apache Hive",
    "Apache Impala",
    "Apache Kafka",
    "Azure PostgreSQL",
    "Big Data File",
    "Bloom Filter",
    "Box",
    "Change Apply",
    "Change Capture",
    "Checksum",
    "Column Export",
    "Column Generator",
    "Column Import",
    "Combine Records",
    "Compare",
    "Complex Flat File",
    "Compress",
    "Copy",
    "Data Rules",
    "Data Service",
    "Data set",
    "DataStax Enterprise",
    "Decode",
    "Denodo",
    "Difference",
    "Distributed Transaction",
    "Dremio",
    "Dropbox",
    "Elasticsearch",
    "Encode",
    "Exasol",
    "Excel",
    "Expand",
    "External Filter",
    "External Source",
    "External Target",
    "FTP",
    "File set",
    "Filter",
    "Funnel",
    "Generic",
    "Generic JDBC",
    "Generic S3",
    "Google BigQuery",
    "Google Cloud Pub/Sub",
    "Google Cloud Storage",
    "Google Looker",
    "Greenplum",
    "HTTP",
    "Head",
    "Hierarchical Data",
    "IBM Cloud Data Engine",
    "IBM Cloud Databases for DataStax",
    "IBM Cloud Databases for MongoDB",
    "IBM Cloud Databases for MySQL",
    "IBM Cloud Databases for PostgreSQL",
    "IBM Cloud Object Storage",
    "IBM Cognos Analytics",
    "IBM Data Virtualization",
    "IBM Data Virtualization Manager for z/OS",
    "IBM Db2",
    "IBM Db2 Big SQL",
    "IBM Db2 Event Store",
    "IBM Db2 Hosted",
    "IBM Db2 Warehouse",
    "IBM Db2 for DataStage",
    "IBM Db2 for i",
    "IBM Db2 for z/OS",
    "IBM Db2 on Cloud",
    "IBM Informix",
    "IBM MQ",
    "IBM Match 360",
    "IBM Netezza Performance Server",
    "IBM Netezza Performance Server for DataStage",
    "IBM Planning Analytics",
    "IBM Product Master",
    "IBM watsonx.data Presto",
    "Investigate",
    "Java Integration",
    "Join",
    "Lookup",
    "Lookup file set",
    "Make Subrecord",
    "Make Vector",
    "MariaDB",
    "Match Frequency",
    "MatchFreq",
    "Merge",
    "Microsoft Azure Blob Storage",
    "Microsoft Azure Cosmos DB",
    "Microsoft Azure Data Lake Storage",
    "Microsoft Azure Databricks",
    "Microsoft Azure File Storage",
    "Microsoft Azure SQL Database",
    "Microsoft Azure Synapse Analytics",
    "Microsoft SQL Server",
    "MinIO",
    "Modify",
    "MongoDB",
    "MySQL",
    "ODBC",
    "One-source Match",
    "Operational Decision Manager",
    "Oracle",
    "Oracle Database for DataStage",
    "Peek",
    "Pivot Enterprise",
    "PostgreSQL",
    "Presto",
    "Promote Subrecord",
    "QSmwi",
    "Remove Duplicates",
    "Rest",
    "Row Generator",
    "SAP ASE",
    "SAP BAPI",
    "SAP Bulk Extract",
    "SAP Delta Extract",
    "SAP HANA",
    "SAP IDoc",
    "SAP IDoc Extract",
    "SAP IQ",
    "SAP OData",
    "Salesforce API for DataStage",
    "Salesforce.com",
    "Sample",
    "Sequential file",
    "SingleStoreDB",
    "Slowly Changing Dimension",
    "Snowflake",
    "Sort",
    "Split Subrecord",
    "Split Vector",
    "Standardize",
    "Storage volume",
    "Stored Procedure",
    "Surrogate Key Generator",
    "Survive",
    "Switch",
    "Tableau",
    "Tail",
    "Teradata",
    "Teradata database for DataStage",
    "Transformer",
    "Two-source Match",
    "Vertica",
    "Wave Generator",
    "Web Service",
    "Write Range Map",
    "XML Input",
    "XML Output",
    "zOSFile",
]
