OP_MAPPINGS = {
    "db2": "db2",
    "db2cloud": "db2cloud",
    "bigsql": "bigsql",
    "db2iseries": "db2iseries",
    "DB2ConnectorPX": "db2fordatastage",
    "dashdb": "db2warehouse",
    "db2zos": "db2zos",
    "RedshiftPX": "amazon_redshift",
    "postgresql-amazon": "amazon_postgresql",
    "oracle-amazon": "amazonrds_oracle",
    "AmazonS3PX": "amazons3",
    "mysql": "mysql",
    "mysql-amazon": "mysql_amazon",
    "datastax": "datastax",
    "cloudobjectstoragePX": "cloud_object_storage",
    "WebSphereMQConnectorPX": "ibm_mq",
    "informix": "informix",
    "match360": "match360",
    "cognos-analytics": "cognos_analytics",
    "volumes": "storage_volume",
    "mongodb-ibmcloud": "mongodb_ibmcloud",
    "dvm": "dvm",
    "lakehouse": "watsonx_data",
    "dv": "dv",
    "tm1odata": "planning_analytics",
    "PxDataSet": "dataset",
    "PxFileSet": "fileset",
    "PxLookupFileSet": "lookupfileset",
    "PxSequentialFile": "sequentialfile",
    "generics3": "generics3",
    "box": "box",
    "elasticsearch": "elasticsearch",
    "dropbox": "dropbox",
    "minio": "minio",
    "ftp": "ftp",
    "tableau": "tableau",
    "looker": "google_looker",
    "hdfs-apache": "hdfs_apache",
    "PxCFF": "complex_flat_file",
    "PxExternalSource": "external_source",
    "PxExternalTarget": "external_target",
    "JavaStagePX": "java_integration",
    "HBaseConnectorPX": "apache_hbase",
    "KafkaConnectorPX": "apache_kafka",
    "HiveConnectorPX": "apache_hive",
    "cassandra": "cassandra",
    "CassandraConnectorPX": "cassandra_datastage",
    "impala": "impala",
    "derby": "derby",
    "bigqueryPX": "bigquery",
    "GoogleCloudStoragePX": "google_cloud_storage",
    "GooglePubSubPX": "google_pub_sub",
    "azureblobstorage": "azure_blob_storage",
    "cosmos": "azure_cosmos",
    "databricks": "azure_databricks",
    "azurefilestorage": "azure_file_storage",
    "AzureDatalakePX": "azuredatalake",
    "azuresql": "azuresql",
    "sqlserver": "sqlserver",
    "sapbulkextract": "sapbulkextract",
    "sapdeltaextract": "sapdeltaextract",
    "saphana": "saphana",
    "sapidoc": "sapidoc",
    "sybaseiq": "sapiq",
    "sapodata": "sapodata",
    "PxBapi": "sapbapi",
    "sybase": "sybase",
    "mysql-compose": "mysql_compose",
    "mariadb": "mariadb",
    "postgresql": "postgresql",
    "postgresql-ibmcloud": "postgresql_ibmcloud",
    "greenplum": "greenplum",
    "JDBCConnectorPX": "jdbc",
    "ODBCConnectorPX": "odbc",
    "oracle": "oracle",
    "dremio": "dremio",
    "mongodb": "mongodb",
    "exasol": "exasol",
    "presto": "presto",
    "TeradataConnectorPX": "teradata_datastage",
    "teradata": "teradata",
    "netezza": "netezza",
    "NetezzaConnectorPX": "netezza_optimized",
    "OracleConnectorPX": "oracle_datastage",
    "PxRest": "rest",
    "denodo": "denodo",
    "vertica": "vertica",
    "SnowflakeConnectorPX": "snowflake",
    "singlestore": "singlestore",
    "SALESFORCEJCConnectorPX": "salesforceapi",
    "salesforce": "salesforce",
    "PxFunnel": "funnel",
    "PxJoin": "join",
    "PxMerge": "merge",
    "PxCopy": "copy",
    "PxCombineRecords": "combine_records",
    "PxMakeVect": "make_vector",
    "PxMakeSubRec": "make_subrecord",
    "PxLookup": "lookup",
    "PxPivot": "pivot",
    "PxAggregator": "aggregator",
    "PxCompare": "compare",
    "PxFilter": "filter",
    "PxSort": "sort",
    "PxRemDup": "remove_duplicates",
    "PxChangeCapture": "change_capture",
    "PxBLM": "bloom_filter",
    "PxDifference": "difference",
    "PxSwitch": "switch",
    "PxExternalFilter": "external_filter",
    "PxModify": "modify",
    "PxSample": "sample",
    "PxChangeApply": "change_apply",
    "PxColumnExport": "column_export",
    "PxChecksum": "checksum",
    "PxEncode": "encode",
    "PxColumnImport": "column_import",
    "PxCompress": "compress",
    "PxExpand": "expand",
    "PxWriteRangeMap": "write_range_map",
    "PxDecode": "decode",
    "PxPeek": "peek",
    "PxRowGenerator": "row_generator",
    "PxColumnGenerator": "column_generator",
    "PxSurrogateKeyGeneratorN": "surrogate_key_generator",
    "PxWaveGenerator": "wave_generator",
    "PxSplitSubRec": "split_subrecord",
    "PxPromoteSubRec": "promote_subrecord",
    "PxHead": "head",
    "PxTail": "tail",
    "PxSplitVect": "split_vector",
    "CTransformerStage": "transformer",
    "http": "http",
    "azuresynapse": "azuresynapse",
    "postgresql-azure": "azure_postgresql",
    "PxXMLInput": "xml_input",
    "XMLStagePX": "hierarchical_data",
    "PxXMLOutput": "xml_output",
    "MatchFrequency": "match_frequency",
    "WSTransformerPX": "web_service",
    "Survive": "survive",
    "Standardize": "standardize",
    "Investigate": "investigate",
    "PxSCD": "slowly_changing_dimension",
    "PxGeneric": "generic",
    "AddressVerification2": "address_verification",
    "qsaviop": "address_verification",
    "IADataRule": "data_rules",
    "UnduplicateMatch": "one_source_match",
    "ReferenceMatch": "two_source_match",
}
