# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from .address_verification import AddressVerificationStage
from .aggregator import AggregatorStage
from .mysql_amazon import AmazonRDSForMysqlStage
from .amazonrds_oracle import AmazonRDSForOracleStage
from .amazon_postgresql import AmazonRDSForPostgresqlStage
from .amazon_redshift import AmazonRedshiftStage
from .amazons3 import AmazonS3Stage
from .cassandra import ApacheCassandraStage
from .cassandra_datastage import ApacheCassandraForDatastageStage
from .derby import ApacheDerbyStage
from .hdfs_apache import ApacheHDFSStage
from .apache_hbase import ApacheHbaseStage
from .apache_hive import ApacheHiveStage
from .impala import ApacheImpalaStage
from .apache_kafka import ApacheKafkaStage
from .azure_postgresql import AzurePostgresqlStage
from .bloom_filter import BloomFilterStage
from .box import BoxStage
from .change_apply import ChangeApplyStage
from .change_capture import ChangeCaptureStage
from .checksum import ChecksumStage
from .column_export import ColumnExportStage
from .column_generator import ColumnGeneratorStage
from .column_import import ColumnImportStage
from .combine_records import CombineRecordsStage
from .compare import CompareStage
from .complex_flat_file import ComplexFlatFileStage
from .compress import CompressStage
from .copy import CopyStage
from .data_rules import DataRulesStage
from .dataset import DataSetStage
from .datastax import DatastaxEnterpriseStage
from .decode import DecodeStage
from .denodo import DenodoStage
from .difference import DifferenceStage
from .dremio import DremioStage
from .dropbox import DropboxStage
from .elasticsearch import ElasticsearchStage
from .encode import EncodeStage
from .exasol import ExasolStage
from .expand import ExpandStage
from .external_filter import ExternalFilterStage
from .external_source import ExternalSourceStage
from .external_target import ExternalTargetStage
from .ftp import FTPStage
from .fileset import FileSetStage
from .filter import FilterStage
from .funnel import FunnelStage
from .generic import GenericStage
from .jdbc import GenericJDBCStage
from .generics3 import GenericS3Stage
from .bigquery import GoogleBigqueryStage
from .google_pub_sub import GoogleCloudPubSubStage
from .google_cloud_storage import GoogleCloudStorageStage
from .google_looker import GoogleLookerStage
from .greenplum import GreenplumStage
from .http import HTTPStage
from .head import HeadStage
from .hierarchical_data import HierarchicalDataStage
from .mongodb_ibmcloud import IBMCloudDatabasesForMongodbStage
from .mysql_compose import IBMCloudDatabasesForMysqlStage
from .postgresql_ibmcloud import IBMCloudDatabasesForPostgresqlStage
from .cloud_object_storage import IBMCloudObjectStorageStage
from .cognos_analytics import IBMCognosAnalyticsStage
from .dv import IBMDataVirtualizationStage
from .dvm import IBMDataVirtualizationManagerForZOSStage
from .db2 import IBMDb2Stage
from .bigsql import IBMDb2BigSQLStage
from .db2fordatastage import IBMDb2ForDatastageStage
from .db2iseries import IBMDb2ForIStage
from .db2zos import IBMDb2ForZOSStage
from .db2cloud import IBMDb2OnCloudStage
from .db2warehouse import IBMDb2WarehouseStage
from .informix import IBMInformixStage
from .ibm_mq import IBMMQStage
from .match360 import IBMMatch360Stage
from .netezza import IBMNetezzaPerformanceServerStage
from .netezza_optimized import IBMNetezzaPerformanceServerForDatastageStage
from .planning_analytics import IBMPlanningAnalyticsStage
from .watsonx_data import IBMWatsonxDataPrestoStage
from .investigate import InvestigateStage
from .java_integration import JavaIntegrationStage
from .join import JoinStage
from .lookup import LookupStage
from .lookupfileset import LookupFileSetStage
from .make_subrecord import MakeSubrecordStage
from .make_vector import MakeVectorStage
from .mariadb import MariadbStage
from .match_frequency import MatchFrequencyStage
from .merge import MergeStage
from .azure_blob_storage import MicrosoftAzureBlobStorageStage
from .azure_cosmos import MicrosoftAzureCosmosDBStage
from .azuredatalake import MicrosoftAzureDataLakeStorageStage
from .azure_databricks import MicrosoftAzureDatabricksStage
from .azure_file_storage import MicrosoftAzureFileStorageStage
from .azuresql import MicrosoftAzureSQLDatabaseStage
from .azuresynapse import MicrosoftAzureSynapseAnalyticsStage
from .sqlserver import MicrosoftSQLServerStage
from .minio import MinioStage
from .modify import ModifyStage
from .mongodb import MongodbStage
from .mysql import MysqlStage
from .odbc import ODBCStage
from .one_source_match import OneSourceMatchStage
from .oracle import OracleStage
from .oracle_datastage import OracleDatabaseForDatastageStage
from .peek import PeekStage
from .pivot import PivotEnterpriseStage
from .postgresql import PostgresqlStage
from .presto import PrestoStage
from .promote_subrecord import PromoteSubrecordStage
from .remove_duplicates import RemoveDuplicatesStage
from .rest import RestStage
from .row_generator import RowGeneratorStage
from .sybase import SAPASEStage
from .sapbapi import SAPBAPIStage
from .sapbulkextract import SAPBulkExtractStage
from .sapdeltaextract import SAPDeltaExtractStage
from .saphana import SAPHANAStage
from .sapiq import SAPIQStage
from .sapidoc import SAPIdocStage
from .sapodata import SAPOdataStage
from .salesforceapi import SalesforceAPIForDatastageStage
from .salesforce import SalesforceComStage
from .sample import SampleStage
from .sequentialfile import SequentialFileStage
from .singlestore import SinglestoredbStage
from .slowly_changing_dimension import SlowlyChangingDimensionStage
from .snowflake import SnowflakeStage
from .sort import SortStage
from .split_subrecord import SplitSubrecordStage
from .split_vector import SplitVectorStage
from .standardize import StandardizeStage
from .storage_volume import StorageVolumeStage
from .surrogate_key_generator import SurrogateKeyGeneratorStage
from .survive import SurviveStage
from .switch import SwitchStage
from .tableau import TableauStage
from .tail import TailStage
from .teradata import TeradataStage
from .teradata_datastage import TeradataDatabaseForDatastageStage
from .transformer import TransformerStage
from .two_source_match import TwoSourceMatchStage
from .vertica import VerticaStage
from .wave_generator import WaveGeneratorStage
from .web_service import WebServiceStage
from .write_range_map import WriteRangeMapStage
from .xml_input import XMLInputStage
from .xml_output import XMLOutputStage


__all__ = [
    'AddressVerificationStage',
    'AggregatorStage',
    'AmazonRDSForMysqlStage',
    'AmazonRDSForOracleStage',
    'AmazonRDSForPostgresqlStage',
    'AmazonRedshiftStage',
    'AmazonS3Stage',
    'ApacheCassandraStage',
    'ApacheCassandraForDatastageStage',
    'ApacheDerbyStage',
    'ApacheHDFSStage',
    'ApacheHbaseStage',
    'ApacheHiveStage',
    'ApacheImpalaStage',
    'ApacheKafkaStage',
    'AzurePostgresqlStage',
    'BloomFilterStage',
    'BoxStage',
    'ChangeApplyStage',
    'ChangeCaptureStage',
    'ChecksumStage',
    'ColumnExportStage',
    'ColumnGeneratorStage',
    'ColumnImportStage',
    'CombineRecordsStage',
    'CompareStage',
    'ComplexFlatFileStage',
    'CompressStage',
    'CopyStage',
    'DataRulesStage',
    'DataSetStage',
    'DatastaxEnterpriseStage',
    'DecodeStage',
    'DenodoStage',
    'DifferenceStage',
    'DremioStage',
    'DropboxStage',
    'ElasticsearchStage',
    'EncodeStage',
    'ExasolStage',
    'ExpandStage',
    'ExternalFilterStage',
    'ExternalSourceStage',
    'ExternalTargetStage',
    'FTPStage',
    'FileSetStage',
    'FilterStage',
    'FunnelStage',
    'GenericStage',
    'GenericJDBCStage',
    'GenericS3Stage',
    'GoogleBigqueryStage',
    'GoogleCloudPubSubStage',
    'GoogleCloudStorageStage',
    'GoogleLookerStage',
    'GreenplumStage',
    'HTTPStage',
    'HeadStage',
    'HierarchicalDataStage',
    'IBMCloudDatabasesForMongodbStage',
    'IBMCloudDatabasesForMysqlStage',
    'IBMCloudDatabasesForPostgresqlStage',
    'IBMCloudObjectStorageStage',
    'IBMCognosAnalyticsStage',
    'IBMDataVirtualizationStage',
    'IBMDataVirtualizationManagerForZOSStage',
    'IBMDb2Stage',
    'IBMDb2BigSQLStage',
    'IBMDb2ForDatastageStage',
    'IBMDb2ForIStage',
    'IBMDb2ForZOSStage',
    'IBMDb2OnCloudStage',
    'IBMDb2WarehouseStage',
    'IBMInformixStage',
    'IBMMQStage',
    'IBMMatch360Stage',
    'IBMNetezzaPerformanceServerStage',
    'IBMNetezzaPerformanceServerForDatastageStage',
    'IBMPlanningAnalyticsStage',
    'IBMWatsonxDataPrestoStage',
    'InvestigateStage',
    'JavaIntegrationStage',
    'JoinStage',
    'LookupStage',
    'LookupFileSetStage',
    'MakeSubrecordStage',
    'MakeVectorStage',
    'MariadbStage',
    'MatchFrequencyStage',
    'MergeStage',
    'MicrosoftAzureBlobStorageStage',
    'MicrosoftAzureCosmosDBStage',
    'MicrosoftAzureDataLakeStorageStage',
    'MicrosoftAzureDatabricksStage',
    'MicrosoftAzureFileStorageStage',
    'MicrosoftAzureSQLDatabaseStage',
    'MicrosoftAzureSynapseAnalyticsStage',
    'MicrosoftSQLServerStage',
    'MinioStage',
    'ModifyStage',
    'MongodbStage',
    'MysqlStage',
    'ODBCStage',
    'OneSourceMatchStage',
    'OracleStage',
    'OracleDatabaseForDatastageStage',
    'PeekStage',
    'PivotEnterpriseStage',
    'PostgresqlStage',
    'PrestoStage',
    'PromoteSubrecordStage',
    'RemoveDuplicatesStage',
    'RestStage',
    'RowGeneratorStage',
    'SAPASEStage',
    'SAPBAPIStage',
    'SAPBulkExtractStage',
    'SAPDeltaExtractStage',
    'SAPHANAStage',
    'SAPIQStage',
    'SAPIdocStage',
    'SAPOdataStage',
    'SalesforceAPIForDatastageStage',
    'SalesforceComStage',
    'SampleStage',
    'SequentialFileStage',
    'SinglestoredbStage',
    'SlowlyChangingDimensionStage',
    'SnowflakeStage',
    'SortStage',
    'SplitSubrecordStage',
    'SplitVectorStage',
    'StandardizeStage',
    'StorageVolumeStage',
    'SurrogateKeyGeneratorStage',
    'SurviveStage',
    'SwitchStage',
    'TableauStage',
    'TailStage',
    'TeradataStage',
    'TeradataDatabaseForDatastageStage',
    'TransformerStage',
    'TwoSourceMatchStage',
    'VerticaStage',
    'WaveGeneratorStage',
    'WebServiceStage',
    'WriteRangeMapStage',
    'XMLInputStage',
    'XMLOutputStage',

]
