# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""This module contains public API to work with Python Generator."""

from ibm_watsonx_data_integration.codegen.generator import PythonGenerator

__all__ = [
    "PythonGenerator",
]
