# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from typing import TYPE_CHECKING

from ibm_watsonx_data_integration.services.datastage.models.flow.dag import StageNode
from ibm_watsonx_data_integration.services.datastage.models.stage_models.match_frequency import match_frequency


if TYPE_CHECKING:
    from ibm_watsonx_data_integration.services.datastage.models.flow.dag import DAG


# from ibm.datastage._framework.components.match_specification import MatchSpecification


class MatchFrequencyStage(StageNode):
    def __init__(self, dag: "DAG", configuration: match_frequency=None, *, label: str = None):
        super().__init__(dag, configuration=configuration, label=label)
        self.configuration: match_frequency = configuration or match_frequency()


    # def set_match_specification(self, match_specification: MatchSpecification):
    #     self.configuration.match_specification = match_specification
    #     self.configuration.specfile = f"{match_specification.name}.MAT"
    #     self.configuration.not_use_specfile = False
    #     self.configuration.workdir = (
    #         f"quality_stage/match_specification/{match_specification.name}"
    #     )
    #     self.configuration.maxfreq = match_specification.MAXFREQ
    #     self.configuration.match_asset_id = match_specification.asset_id
    #     self.configuration.match_type = match_specification.MATCHTYPE
