CONN_MAPPINGS = {
    "amazons3": "amazons3",
    "AmazonS3PX": "amazons3",
    "apachekafka": "apache_kafka",
    "KafkaConnectorPX": "apache_kafka",
    "awsglue": "awsglue",
    "azureblobstorage": "azure_blob_storage",
    "azuredatalake": "azuredatalake",
    "AzureDatalakePX": "azuredatalake",
    "azurefilestorage": "azure_file_storage",
    "azuresql": "azuresql",
    "azuresynapse": "azuresynapse",
    "bigquery": "bigquery",
    "bigqueryPX": "bigquery",
    "bigsql": "bigsql",
    "bluemixcloudobjectstorage": "bluemixcloudobjectstorage",
    "box": "box",
    "cassandra": "cassandra",
    "CassandraConnectorPX": "cassandra",
    "cassandra-datastage": "cassandra_datastage",
    "cloudant": "cloudant",
    "cloudobjectstorage": "cloud_object_storage",
    "cloudobjectstoragePX": "cloud_object_storage",
    "cognos-analytics": "cognos_analytics",
    "cognos-query-service": "cognos-query-service",
    "cosmos": "azure_cosmos",
    "custom-noop": "custom-noop",
    "dashdb": "db2warehouse",
    "databricks": "azure_databricks",
    "datastax": "datastax",
    "db2": "db2",
    "DB2ConnectorPX": "db2",
    "db2-datastage": "db2fordatastage",
    "db2cloud": "db2cloud",
    "db2iseries": "db2iseries",
    "db2zos": "db2zos",
    "denodo": "denodo",
    "derby": "derby",
    "dremio": "dremio",
    "dropbox": "dropbox",
    "dv": "dv",
    "dvm": "dvm",
    "elastic-cloud": "elastic-cloud",
    "elasticsearch": "elasticsearch",
    "exasol": "exasol",
    "ftp": "ftp",
    "genericjdbc": "jdbc",
    "JDBCConnectorPX": "jdbc",
    "generics3": "generics3",
    "github": "github",
    "googlecloudstorage": "google_cloud_storage",
    "GoogleCloudStoragePX": "google_cloud_storage",
    "googlepubsub": "google_pub_sub",
    "GooglePubSubPX": "google_pub_sub",
    "greenplum": "greenplum",
    "hbase-datastage": "apache_hbase",
    "HBaseConnectorPX": "apache_hbase",
    "hdfs-apache": "hdfs_apache",
    "hive": "apache_hive",
    "HiveConnectorPX": "apache_hive",
    "http": "http",
    "ibm-datastage-cp4d": "ibm-datastage-cp4d",
    "ibm-infosphere-datastage": "ibm-infosphere-datastage",
    "impala": "impala",
    "informix": "informix",
    "lakehouse": "watsonx_data",
    "looker": "google_looker",
    "mariadb": "mariadb",
    "match360": "match360",
    "microstrategy": "microstrategy",
    "milvus": "milvus",
    "milvuswxd": "milvuswxd",
    "minio": "minio",
    "mongodb": "mongodb",
    "mongodb-ibmcloud": "mongodb_ibmcloud",
    "mysql": "mysql",
    "mysql-amazon": "mysql_amazon",
    "mysql-ibmcloud": "mysql_compose",
    "mysql-compose": "mysql_compose",
    "nearbysearch": "nearbysearch",
    "netezza": "netezza",
    "NetezzaConnectorPX": "netezza",
    "netezza-datastage": "netezza_optimized",
    "odata": "odata",
    "odbc-datastage": "odbc",
    "ODBCConnectorPX": "odbc",
    "openlineage": "openlineage",
    "oracle": "oracle",
    "OracleConnectorPX": "oracle",
    "oracle-amazon": "amazonrds_oracle",
    "oracle-datastage": "oracle_datastage",
    "postgresql": "postgresql",
    "postgresql-amazon": "amazon_postgresql",
    "postgresql-azure": "azure_postgresql",
    "postgresql-ibmcloud": "postgresql_ibmcloud",
    "powerbi-azure": "powerbi-azure",
    "presto": "presto",
    "redshift": "amazon_redshift",
    "RedshiftPX": "amazon_redshift",
    "restgeneric": "restgeneric",
    "riakkv": "riakkv",
    "salesforce": "salesforce",
    "salesforce-datastage": "salesforceapi",
    "SALESFORCEJCConnectorPX": "salesforceapi",
    "sapbapi": "sapbapi",
    "sapbulkextract": "sapbulkextract",
    "sapdeltaextract": "sapdeltaextract",
    "saphana": "saphana",
    "sapidoc": "sapidoc",
    "sapodata": "sapodata",
    "singlestore": "singlestore",
    "snowflake": "snowflake",
    "SnowflakeConnectorPX": "snowflake",
    "sqlserver": "sqlserver",
    "sybase": "sybase",
    "sybaseiq": "sapiq",
    "tableau": "tableau",
    "teradata": "teradata",
    "TeradataConnectorPX": "teradata",
    "teradata-datastage": "teradata_datastage",
    "tm1odata": "planning_analytics",
    "twitter": "twitter",
    "vertica": "vertica",
    "volumes": "storage_volume",
    "webspheremq-datastage": "ibm_mq",
    "WebSphereMQConnectorPX": "ibm_mq",
}
