# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from enum import Enum

class StageTypeEnum(Enum):
    """
    Enum representing different types of stages in a DataStage flow. Each value corresponds to a specific stage class
    that can be instantiated.
    """

    ADDRESS_VERIFICATION = "Address Verification"
    AGGREGATOR = "Aggregator"
    AMAZON_RDS_FOR_MYSQL = "Amazon RDS for MySQL"
    AMAZON_RDS_FOR_ORACLE = "Amazon RDS for Oracle"
    AMAZON_RDS_FOR_POSTGRESQL = "Amazon RDS for PostgreSQL"
    AMAZON_REDSHIFT = "Amazon Redshift"
    AMAZON_S3 = "Amazon S3"
    APACHE_CASSANDRA = "Apache Cassandra"
    APACHE_CASSANDRA_FOR_DATASTAGE = "Apache Cassandra for DataStage"
    APACHE_DERBY = "Apache Derby"
    APACHE_HBASE = "Apache HBase"
    APACHE_HDFS = "Apache HDFS"
    APACHE_HIVE = "Apache Hive"
    APACHE_IMPALA = "Apache Impala"
    APACHE_KAFKA = "Apache Kafka"
    AZURE_POSTGRESQL = "Azure PostgreSQL"
    BLOOM_FILTER = "Bloom Filter"
    BOX = "Box"
    CHANGE_APPLY = "Change Apply"
    CHANGE_CAPTURE = "Change Capture"
    CHECKSUM = "Checksum"
    COLUMN_EXPORT = "Column Export"
    COLUMN_GENERATOR = "Column Generator"
    COLUMN_IMPORT = "Column Import"
    COMBINE_RECORDS = "Combine Records"
    COMPARE = "Compare"
    COMPLEX_FLAT_FILE = "Complex Flat File"
    COMPRESS = "Compress"
    COPY = "Copy"
    DATASTAX_ENTERPRISE = "DataStax Enterprise"
    DATA_RULES = "Data Rules"
    DATA_SET = "Data set"
    DECODE = "Decode"
    DENODO = "Denodo"
    DIFFERENCE = "Difference"
    DREMIO = "Dremio"
    DROPBOX = "Dropbox"
    ELASTICSEARCH = "Elasticsearch"
    ENCODE = "Encode"
    EXASOL = "Exasol"
    EXPAND = "Expand"
    EXTERNAL_FILTER = "External Filter"
    EXTERNAL_SOURCE = "External Source"
    EXTERNAL_TARGET = "External Target"
    FILE_SET = "File set"
    FILTER = "Filter"
    FTP = "FTP"
    FUNNEL = "Funnel"
    GENERIC = "Generic"
    GENERIC_JDBC = "Generic JDBC"
    GENERIC_S3 = "Generic S3"
    GOOGLE_BIGQUERY = "Google BigQuery"
    GOOGLE_CLOUD_PUBSUB = "Google Cloud Pub/Sub"
    GOOGLE_CLOUD_STORAGE = "Google Cloud Storage"
    GOOGLE_LOOKER = "Google Looker"
    GREENPLUM = "Greenplum"
    HEAD = "Head"
    HIERARCHICAL_DATA = "Hierarchical Data"
    HTTP = "HTTP"
    IBM_CLOUD_DATABASES_FOR_MONGODB = "IBM Cloud Databases for MongoDB"
    IBM_CLOUD_DATABASES_FOR_MYSQL = "IBM Cloud Databases for MySQL"
    IBM_CLOUD_DATABASES_FOR_POSTGRESQL = "IBM Cloud Databases for PostgreSQL"
    IBM_CLOUD_OBJECT_STORAGE = "IBM Cloud Object Storage"
    IBM_COGNOS_ANALYTICS = "IBM Cognos Analytics"
    IBM_DATA_VIRTUALIZATION = "IBM Data Virtualization"
    IBM_DATA_VIRTUALIZATION_MANAGER_FOR_ZOS = "IBM Data Virtualization Manager for z/OS"
    IBM_DB2 = "IBM Db2"
    IBM_DB2_BIG_SQL = "IBM Db2 Big SQL"
    IBM_DB2_FOR_DATASTAGE = "IBM Db2 for DataStage"
    IBM_DB2_FOR_I = "IBM Db2 for i"
    IBM_DB2_FOR_ZOS = "IBM Db2 for z/OS"
    IBM_DB2_ON_CLOUD = "IBM Db2 on Cloud"
    IBM_DB2_WAREHOUSE = "IBM Db2 Warehouse"
    IBM_INFORMIX = "IBM Informix"
    IBM_MATCH_360 = "IBM Match 360"
    IBM_MQ = "IBM MQ"
    IBM_NETEZZA_PERFORMANCE_SERVER = "IBM Netezza Performance Server"
    IBM_NETEZZA_PERFORMANCE_SERVER_FOR_DATASTAGE = "IBM Netezza Performance Server for DataStage"
    IBM_PLANNING_ANALYTICS = "IBM Planning Analytics"
    IBM_WATSONX_DATA_PRESTO = "IBM watsonx.data Presto"
    INVESTIGATE = "Investigate"
    JAVA_INTEGRATION = "Java Integration"
    JOIN = "Join"
    LOOKUP = "Lookup"
    LOOKUP_FILE_SET = "Lookup file set"
    MAKE_SUBRECORD = "Make Subrecord"
    MAKE_VECTOR = "Make Vector"
    MARIADB = "MariaDB"
    MATCH_FREQUENCY = "Match Frequency"
    MERGE = "Merge"
    MICROSOFT_AZURE_BLOB_STORAGE = "Microsoft Azure Blob Storage"
    MICROSOFT_AZURE_COSMOS_DB = "Microsoft Azure Cosmos DB"
    MICROSOFT_AZURE_DATABRICKS = "Microsoft Azure Databricks"
    MICROSOFT_AZURE_DATA_LAKE_STORAGE = "Microsoft Azure Data Lake Storage"
    MICROSOFT_AZURE_FILE_STORAGE = "Microsoft Azure File Storage"
    MICROSOFT_AZURE_SQL_DATABASE = "Microsoft Azure SQL Database"
    MICROSOFT_AZURE_SYNAPSE_ANALYTICS = "Microsoft Azure Synapse Analytics"
    MICROSOFT_SQL_SERVER = "Microsoft SQL Server"
    MINIO = "MinIO"
    MODIFY = "Modify"
    MONGODB = "MongoDB"
    MYSQL = "MySQL"
    ODBC = "ODBC"
    ONE_SOURCE_MATCH = "One-source Match"
    ORACLE = "Oracle"
    ORACLE_DATABASE_FOR_DATASTAGE = "Oracle Database for DataStage"
    PEEK = "Peek"
    PIVOT_ENTERPRISE = "Pivot Enterprise"
    POSTGRESQL = "PostgreSQL"
    PRESTO = "Presto"
    PROMOTE_SUBRECORD = "Promote Subrecord"
    REMOVE_DUPLICATES = "Remove Duplicates"
    REST = "Rest"
    ROW_GENERATOR = "Row Generator"
    SALESFORCE_API_FOR_DATASTAGE = "Salesforce API for DataStage"
    SALESFORCE_COM = "Salesforce.com"
    SAMPLE = "Sample"
    SAP_ASE = "SAP ASE"
    SAP_BAPI = "SAP BAPI"
    SAP_BULK_EXTRACT = "SAP Bulk Extract"
    SAP_DELTA_EXTRACT = "SAP Delta Extract"
    SAP_HANA = "SAP HANA"
    SAP_IDOC = "SAP IDoc"
    SAP_IQ = "SAP IQ"
    SAP_ODATA = "SAP OData"
    SEQUENTIAL_FILE = "Sequential file"
    SINGLESTOREDB = "SingleStoreDB"
    SLOWLY_CHANGING_DIMENSION = "Slowly Changing Dimension"
    SNOWFLAKE = "Snowflake"
    SORT = "Sort"
    SPLIT_SUBRECORD = "Split Subrecord"
    SPLIT_VECTOR = "Split Vector"
    STANDARDIZE = "Standardize"
    STORAGE_VOLUME = "Storage volume"
    SURROGATE_KEY_GENERATOR = "Surrogate Key Generator"
    SURVIVE = "Survive"
    SWITCH = "Switch"
    TABLEAU = "Tableau"
    TAIL = "Tail"
    TERADATA = "Teradata"
    TERADATA_DATABASE_FOR_DATASTAGE = "Teradata database for DataStage"
    TRANSFORMER = "Transformer"
    TWO_SOURCE_MATCH = "Two-source Match"
    VERTICA = "Vertica"
    WAVE_GENERATOR = "Wave Generator"
    WEB_SERVICE = "Web Service"
    WRITE_RANGE_MAP = "Write Range Map"
    XML_INPUT = "XML Input"
    XML_OUTPUT = "XML Output"
