# IBM Confidential

"""This module contains the API for the DataStage CPD Service."""

from ibm_watsonx_data_integration.services.datastage.api.datastage_flow_api import DatastageFlowApiClient

__all__ = [
    "DatastageFlowApiClient",
]
