# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from typing import TYPE_CHECKING

from ibm_watsonx_data_integration.services.datastage.models.flow.dag import StageNode
from ibm_watsonx_data_integration.services.datastage.models.stage_models.transformer import transformer


if TYPE_CHECKING:
    from ibm_watsonx_data_integration.services.datastage.models.flow.dag import DAG


import os
# from ibm.datastage._framework.components.function_library import FunctionLibrary


class TransformerStage(StageNode):
    def __init__(self, dag: "DAG", configuration: transformer=None, *, label: str = None):
        super().__init__(dag, configuration=configuration, label=label)
        self.configuration: transformer = configuration or transformer()


    # def set_function_library(self, function_library: FunctionLibrary):
    #    self.configuration.function_library = function_library
    #    self.configuration.function_library_asset_name = function_library.name
    #    self.configuration.function_library_path = function_library.library_path
