# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

import ibm_watsonx_data_integration.services.datastage.models.stages as stage

# STAGE_NAMES is a dictionary mapping string stage names to their corresponding StageNode classes.
STAGE_NAMES = {
    "Address Verification": stage.AddressVerificationStage,
    "Aggregator": stage.AggregatorStage,
    "Amazon RDS for MySQL": stage.AmazonRDSForMysqlStage,
    "Amazon RDS for Oracle": stage.AmazonRDSForOracleStage,
    "Amazon RDS for PostgreSQL": stage.AmazonRDSForPostgresqlStage,
    "Amazon Redshift": stage.AmazonRedshiftStage,
    "Amazon S3": stage.AmazonS3Stage,
    "Apache Cassandra": stage.ApacheCassandraStage,
    "Apache Cassandra for DataStage": stage.ApacheCassandraForDatastageStage,
    "Apache Derby": stage.ApacheDerbyStage,
    "Apache HDFS": stage.ApacheHDFSStage,
    "Apache Hive": stage.ApacheHiveStage,
    "Apache Impala": stage.ApacheImpalaStage,
    "Apache Kafka": stage.ApacheKafkaStage,
    "Azure PostgreSQL": stage.AzurePostgresqlStage,
    "Bloom Filter": stage.BloomFilterStage,
    "Box": stage.BoxStage,
    "Change Apply": stage.ChangeApplyStage,
    "Change Capture": stage.ChangeCaptureStage,
    "Checksum": stage.ChecksumStage,
    "Column Export": stage.ColumnExportStage,
    "Column Generator": stage.ColumnGeneratorStage,
    "Column Import": stage.ColumnImportStage,
    "Combine Records": stage.CombineRecordsStage,
    "Compare": stage.CompareStage,
    "Complex Flat File": stage.ComplexFlatFileStage,
    "Compress": stage.CompressStage,
    "Copy": stage.CopyStage,
    "Data Rules": stage.DataRulesStage,
    "Data set": stage.DataSetStage,
    "DataStax Enterprise": stage.DatastaxEnterpriseStage,
    "Decode": stage.DecodeStage,
    "Denodo": stage.DenodoStage,
    "Difference": stage.DifferenceStage,
    "Dremio": stage.DremioStage,
    "Dropbox": stage.DropboxStage,
    "Elasticsearch": stage.ElasticsearchStage,
    "Encode": stage.EncodeStage,
    "Expand": stage.ExpandStage,
    "External Filter": stage.ExternalFilterStage,
    "External Source": stage.ExternalSourceStage,
    "External Target": stage.ExternalTargetStage,
    "FTP": stage.FTPStage,
    "File set": stage.FileSetStage,
    "Filter": stage.FilterStage,
    "Funnel": stage.FunnelStage,
    "Generic": stage.GenericStage,
    "Generic JDBC": stage.GenericJDBCStage,
    "Generic S3": stage.GenericS3Stage,
    "Google BigQuery": stage.GoogleBigqueryStage,
    "Google Cloud Pub/Sub": stage.GoogleCloudPubSubStage,
    "Google Cloud Storage": stage.GoogleCloudStorageStage,
    "Google Looker": stage.GoogleLookerStage,
    "Greenplum": stage.GreenplumStage,
    "HTTP": stage.HTTPStage,
    "Head": stage.HeadStage,
    "Hierarchical Data": stage.HierarchicalDataStage,
    "IBM Cloud Databases for MongoDB": stage.IBMCloudDatabasesForMongodbStage,
    "IBM Cloud Databases for PostgreSQL": stage.IBMCloudDatabasesForPostgresqlStage,
    "IBM Cloud Object Storage": stage.IBMCloudObjectStorageStage,
    "IBM Cognos Analytics": stage.IBMCognosAnalyticsStage,
    "IBM Data Virtualization": stage.IBMDataVirtualizationStage,
    "IBM Data Virtualization Manager for z/OS": stage.IBMDataVirtualizationManagerForZOSStage,
    "IBM Db2": stage.IBMDb2Stage,
    "IBM Db2 Big SQL": stage.IBMDb2BigSQLStage,
    "IBM Db2 Warehouse": stage.IBMDb2WarehouseStage,
    "IBM Db2 for DataStage": stage.IBMDb2ForDatastageStage,
    "IBM Db2 for i": stage.IBMDb2ForIStage,
    "IBM Db2 for z/OS": stage.IBMDb2ForZOSStage,
    "IBM Db2 on Cloud": stage.IBMDb2OnCloudStage,
    "IBM Informix": stage.IBMInformixStage,
    "IBM MQ": stage.IBMMQStage,
    "IBM Match 360": stage.IBMMatch360Stage,
    "IBM Netezza Performance Server": stage.IBMNetezzaPerformanceServerStage,
    "IBM Netezza Performance Server for DataStage": stage.IBMNetezzaPerformanceServerForDatastageStage,
    "IBM Planning Analytics": stage.IBMPlanningAnalyticsStage,
    "IBM watsonx.data Presto": stage.IBMWatsonxDataPrestoStage,
    "Investigate": stage.InvestigateStage,
    "Java Integration": stage.JavaIntegrationStage,
    "Join": stage.JoinStage,
    "Lookup": stage.LookupStage,
    "Lookup file set": stage.LookupFileSetStage,
    "Make Subrecord": stage.MakeSubrecordStage,
    "Make Vector": stage.MakeVectorStage,
    "MariaDB": stage.MariadbStage,
    "Match Frequency": stage.MatchFrequencyStage,
    "Merge": stage.MergeStage,
    "Microsoft Azure Blob Storage": stage.MicrosoftAzureBlobStorageStage,
    "Microsoft Azure Cosmos DB": stage.MicrosoftAzureCosmosDBStage,
    "Microsoft Azure Data Lake Storage": stage.MicrosoftAzureDataLakeStorageStage,
    "Microsoft Azure Databricks": stage.MicrosoftAzureDatabricksStage,
    "Microsoft Azure File Storage": stage.MicrosoftAzureFileStorageStage,
    "Microsoft Azure SQL Database": stage.MicrosoftAzureSQLDatabaseStage,
    "Microsoft Azure Synapse Analytics": stage.MicrosoftAzureSynapseAnalyticsStage,
    "Microsoft SQL Server": stage.MicrosoftSQLServerStage,
    "Modify": stage.ModifyStage,
    "MongoDB": stage.MongodbStage,
    "MySQL": stage.MysqlStage,
    "ODBC": stage.ODBCStage,
    "One-source Match": stage.OneSourceMatchStage,
    "Oracle": stage.OracleStage,
    "Oracle Database for DataStage": stage.OracleDatabaseForDatastageStage,
    "Peek": stage.PeekStage,
    "Pivot Enterprise": stage.PivotEnterpriseStage,
    "PostgreSQL": stage.PostgresqlStage,
    "Presto": stage.PrestoStage,
    "Promote Subrecord": stage.PromoteSubrecordStage,
    "Remove Duplicates": stage.RemoveDuplicatesStage,
    "Rest": stage.RestStage,
    "Row Generator": stage.RowGeneratorStage,
    "SAP ASE": stage.SAPASEStage,
    "SAP IQ": stage.SAPIQStage,
    "SAP OData": stage.SAPOdataStage,
    "Salesforce API for DataStage": stage.SalesforceAPIForDatastageStage,
    "Salesforce.com": stage.SalesforceComStage,
    "Sample": stage.SampleStage,
    "Sequential file": stage.SequentialFileStage,
    "SingleStoreDB": stage.SinglestoredbStage,
    "Slowly Changing Dimension": stage.SlowlyChangingDimensionStage,
    "Snowflake": stage.SnowflakeStage,
    "Sort": stage.SortStage,
    "Split Subrecord": stage.SplitSubrecordStage,
    "Split Vector": stage.SplitVectorStage,
    "Standardize": stage.StandardizeStage,
    "Surrogate Key Generator": stage.SurrogateKeyGeneratorStage,
    "Survive": stage.SurviveStage,
    "Switch": stage.SwitchStage,
    "Tableau": stage.TableauStage,
    "Tail": stage.TailStage,
    "Teradata": stage.TeradataStage,
    "Teradata database for DataStage": stage.TeradataDatabaseForDatastageStage,
    "Transformer": stage.TransformerStage,
    "Two-source Match": stage.TwoSourceMatchStage,
    "Vertica": stage.VerticaStage,
    "Wave Generator": stage.WaveGeneratorStage,
    "Web Service": stage.WebServiceStage,
    "Write Range Map": stage.WriteRangeMapStage,
    "XML Input": stage.XMLInputStage,
    "XML Output": stage.XMLOutputStage,
}
