# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

LABEL_MAPPINGS: dict[str, str] = {
    "address_verification": "Address Verification",
    "aggregator": "Aggregator",
    "amazon_postgresql": "Amazon RDS for PostgreSQL",
    "amazon_redshift": "Amazon Redshift",
    "amazonrds_oracle": "Amazon RDS for Oracle",
    "amazons3": "Amazon S3",
    "apache_hbase": "Apache HBase",
    "apache_hive": "Apache Hive",
    "apache_kafka": "Apache Kafka",
    "azure_blob_storage": "Microsoft Azure Blob Storage",
    "azure_cosmos": "Microsoft Azure Cosmos DB",
    "azure_databricks": "Microsoft Azure Databricks",
    "azure_file_storage": "Microsoft Azure File Storage",
    "azure_postgresql": "Azure PostgreSQL",
    "azuredatalake": "Microsoft Azure Data Lake Storage",
    "azuresql": "Microsoft Azure SQL Database",
    "azuresynapse": "Microsoft Azure Synapse Analytics",
    "bigquery": "Google BigQuery",
    "bigsql": "IBM Db2 Big SQL",
    "bloom_filter": "Bloom Filter",
    "box": "Box",
    "cassandra": "Apache Cassandra",
    "cassandra_datastage": "Apache Cassandra for DataStage",
    "change_apply": "Change Apply",
    "change_capture": "Change Capture",
    "checksum": "Checksum",
    "cloud_object_storage": "IBM Cloud Object Storage",
    "cognos_analytics": "IBM Cognos Analytics",
    "column_export": "Column Export",
    "column_generator": "Column Generator",
    "column_import": "Column Import",
    "combine_records": "Combine Records",
    "compare": "Compare",
    "complex_flat_file": "Complex Flat File",
    "compress": "Compress",
    "copy": "Copy",
    "data_rules": "Data Rules",
    "dataset": "Data set",
    "datastax": "DataStax Enterprise",
    "db2": "IBM Db2",
    "db2cloud": "IBM Db2 on Cloud",
    "db2fordatastage": "IBM Db2 for DataStage",
    "db2iseries": "IBM Db2 for i",
    "db2warehouse": "IBM Db2 Warehouse",
    "db2zos": "IBM Db2 for z/OS",
    "decode": "Decode",
    "denodo": "Denodo",
    "derby": "Apache Derby",
    "difference": "Difference",
    "dremio": "Dremio",
    "dropbox": "Dropbox",
    "dv": "IBM Data Virtualization",
    "dvm": "IBM Data Virtualization Manager for z/OS",
    "elasticsearch": "Elasticsearch",
    "encode": "Encode",
    "exasol": "Exasol",
    "expand": "Expand",
    "external_filter": "External Filter",
    "external_source": "External Source",
    "external_target": "External Target",
    "fileset": "File set",
    "filter": "Filter",
    "ftp": "FTP",
    "funnel": "Funnel",
    "generic": "Generic",
    "generics3": "Generic S3",
    "google_cloud_storage": "Google Cloud Storage",
    "google_looker": "Google Looker",
    "google_pub_sub": "Google Cloud Pub/Sub",
    "greenplum": "Greenplum",
    "hdfs_apache": "Apache HDFS",
    "head": "Head",
    "hierarchical_data": "Hierarchical Data",
    "http": "HTTP",
    "ibm_mq": "IBM MQ",
    "impala": "Apache Impala",
    "informix": "IBM Informix",
    "investigate": "Investigate",
    "java_integration": "Java Integration",
    "jdbc": "Generic JDBC",
    "join": "Join",
    "lookup": "Lookup",
    "lookupfileset": "Lookup file set",
    "make_subrecord": "Make Subrecord",
    "make_vector": "Make Vector",
    "mariadb": "MariaDB",
    "match360": "IBM Match 360",
    "match_frequency": "Match Frequency",
    "merge": "Merge",
    "minio": "MinIO",
    "modify": "Modify",
    "mongodb": "MongoDB",
    "mongodb_ibmcloud": "IBM Cloud Databases for MongoDB",
    "mysql": "MySQL",
    "mysql_amazon": "Amazon RDS for MySQL",
    "mysql_compose": "IBM Cloud Databases for MySQL",
    "netezza": "IBM Netezza Performance Server",
    "netezza_optimized": "IBM Netezza Performance Server for DataStage",
    "odbc": "ODBC",
    "one_source_match": "One-source Match",
    "oracle": "Oracle",
    "oracle_datastage": "Oracle Database for DataStage",
    "peek": "Peek",
    "pivot": "Pivot Enterprise",
    "planning_analytics": "IBM Planning Analytics",
    "postgresql": "PostgreSQL",
    "postgresql_ibmcloud": "IBM Cloud Databases for PostgreSQL",
    "presto": "Presto",
    "promote_subrecord": "Promote Subrecord",
    "remove_duplicates": "Remove Duplicates",
    "rest": "Rest",
    "row_generator": "Row Generator",
    "salesforce": "Salesforce.com",
    "salesforceapi": "Salesforce API for DataStage",
    "sample": "Sample",
    "sapbapi": "SAP BAPI",
    "sapbulkextract": "SAP Bulk Extract",
    "sapdeltaextract": "SAP Delta Extract",
    "saphana": "SAP HANA",
    "sapidoc": "SAP IDoc",
    "sapiq": "SAP IQ",
    "sapodata": "SAP OData",
    "sequentialfile": "Sequential file",
    "singlestore": "SingleStoreDB",
    "slowly_changing_dimension": "Slowly Changing Dimension",
    "snowflake": "Snowflake",
    "sort": "Sort",
    "split_subrecord": "Split Subrecord",
    "split_vector": "Split Vector",
    "sqlserver": "Microsoft SQL Server",
    "standardize": "Standardize",
    "storage_volume": "Storage volume",
    "surrogate_key_generator": "Surrogate Key Generator",
    "survive": "Survive",
    "switch": "Switch",
    "sybase": "SAP ASE",
    "tableau": "Tableau",
    "tail": "Tail",
    "teradata": "Teradata",
    "teradata_datastage": "Teradata database for DataStage",
    "transformer": "Transformer",
    "two_source_match": "Two-source Match",
    "vertica": "Vertica",
    "watsonx_data": "IBM watsonx.data Presto",
    "wave_generator": "Wave Generator",
    "web_service": "Web Service",
    "write_range_map": "Write Range Map",
    "xml_input": "XML Input",
    "xml_output": "XML Output",
}
