# IBM Confidential
# PID 5900-BAF
# Copyright StreamSets Inc., an IBM Company 2025

"""This package contains API Clients, models and abstractions for WatsonX Data Integration Services."""

from ibm_watsonx_data_integration.__version__ import __version__  # noqa: F401
from ibm_watsonx_data_integration.platform import Platform

__all__ = ["Platform"]
