# -------------------------------------------------------------------
# This file was auto-generated. Do not edit directly.
# Generated by: StageGeneratorV2
# -------------------------------------------------------------------

from typing import TYPE_CHECKING

from ibm_watsonx_data_integration.services.datastage.models.flow.dag import StageNode
from ibm_watsonx_data_integration.services.datastage.models.stage_models.column_export import column_export


if TYPE_CHECKING:
    from ibm_watsonx_data_integration.services.datastage.models.flow.dag import DAG



class ColumnExportStage(StageNode):
    def __init__(self, dag: "DAG", configuration: column_export=None, *, label: str = None):
        super().__init__(dag, configuration=configuration, label=label)
        self.configuration: column_export = configuration or column_export()
